import { Picker } from 'emoji-picker-element';
import("../utils.js").then((utils) => {
  /**
   * Custom emoji picker.
   * Replaces the default drawer with a custom component.
   */
  converse.plugins.add("sib-emoji-picker", {
    dependencies: ["converse-emoji"],
    initialize() {
      const { html } = converse.env;
      const _converse = this._converse;
      const { api, __ } = _converse;

      /**
       * @type {Picker|HTMLElement}
       */
      let picker;

      /**
       * Force rendering of the emoji picker inside the dropdown.
       * Note: Without this mixin, the IndexedDB can't be loaded for some reason.
       */
      const EmojiDropdownMixin = {
        render() {
          return html` <div class="dropup">
            <button
              class="toggle-emojis"
              title="${__("Insert emojis")}"
              data-toggle="dropdown"
              aria-haspopup="true"
              aria-expanded="false"
            >
              <converse-icon
                class="fa fa-smile"
                path-prefix="${api.settings.get("assets_path")}"
                size="1em"
              ></converse-icon>
            </button>
            <div class="dropdown-menu"></div>
          </div>`;
        },

        /**
         * Create a new picker and attach it to the DOM.
         * The emoji-picker module does not support shadow dom reassignment,
         * therefore breaking the component itself.
         */
        async showMenu() {
          // Remove any existing pickers
          // Happens when the room is switched with the dropdown opened
          // TODO: Add a listener on model change (hidden prop) to automatically hide the menu
          if (picker) {
            picker.remove();
          }

          picker = new Picker();
          picker.classList.add("light");

          picker.addEventListener("emoji-click", (ev) => {
            if (utils.isTouchDevice()) {
              // Close the dropdown once we choose one on mobile devices
              this.hideMenu();
            }
            this.insertIntoTextArea(ev.detail.unicode);
          });

          this.menu.appendChild(picker);

          await _showMenu.apply(this, arguments);

          setTimeout(() => {
            // Focus the search input afterwards
            picker?.shadowRoot?.querySelector(".search")?.focus();
          });
        },

        /**
         * Hide the menu.
         * Detaches the picker from the element.
         */
        hideMenu() {
          _hideMenu.apply(this, arguments);
          if (picker) {
            picker.remove();
          }
        },

        /**
         * Insert the given value into the text area.
         * @see EmojiPicker.insertIntoTextArea
         * @param {string} value
         */
        insertIntoTextArea(value) {
          const autocompleting = this.model.get("autocompleting");
          const ac_position = this.model.get("ac_position");
          this.chatview.insertIntoTextArea(
            value,
            autocompleting,
            false,
            ac_position
          );
          this.model.set({
            autocompleting: null,
            query: "",
            ac_position: null,
          });
        },
      };

      /**
       * @type {EmojiDropdown|HTMLElement}
       */
      const EmojiDropdown = api.elements.registry["converse-emoji-dropdown"];
      const _showMenu = EmojiDropdown.prototype.showMenu;
      const _hideMenu = EmojiDropdown.prototype.hideMenu;

      // Apply mixin
      Object.assign(EmojiDropdown.prototype, EmojiDropdownMixin);
    },
  });
});
